@@ -7,7 +7,7 @@ from gridsource.validation import ValidatorMixin
 
 __author__ = """Nicolas Cordier"""
 __email__ = "nicolas.cordier@numeric-gmbh.ch"
-__version__ = "0.18.0"
+__version__ = "0.25.0"
 
 
 class _Base:
@@ -18,6 +18,13 @@ class _Base:
             if hasattr(self, f):
                 getattr(self, f)()
 
+    def get(self, tabname=None, attr="_data"):
+        if not attr.startswith("_"):
+            attr = "_" + attr
+        if not tabname:
+            raise AttributeError(f"`tabname` should be one of {set(self._data.keys())}")
+        return getattr(self, attr)[tabname]
+
 
 class Data(_Base, IOMixin, ValidatorMixin):
     pass
