@@ -28,7 +28,7 @@ def version(args=None):
 
 
 def format(args):
-    """convert file inplace into its own extension"""
+    """export file inplace into its own extension"""
     data = IOData()
     data.read(args.input, engine=args.engine)
     data.to(args.input)
@@ -36,8 +36,8 @@ def format(args):
     return 0
 
 
-def convert(args):
-    """convertion command processor"""
+def export(args):
+    """exportion command processor"""
     if args.schema is None:
         # create a non-validating IO class only
         data = IOData()
@@ -100,13 +100,13 @@ def parse_args(args):
 
     subparsers = parser.add_subparsers(help="sub-command help")
     # ========================================================================
-    # parser_converter
+    # parser_exporter
     # ========================================================================
-    parser_convert = subparsers.add_parser("convert", help="convert")
-    parser_convert.add_argument("input", type=str, help="path to input file")
-    parser_convert.add_argument("-o", "--output", type=str, help="path to output file")
-    parser_convert.add_argument("-s", "--schema", type=str, help="validation schema")
-    parser_convert.add_argument(
+    parser_export = subparsers.add_parser("export", help="export")
+    parser_export.add_argument("input", type=str, help="path to input file")
+    parser_export.add_argument("-o", "--output", type=str, help="path to output file")
+    parser_export.add_argument("-s", "--schema", type=str, help="validation schema")
+    parser_export.add_argument(
         "-e",
         "--engine",
         type=str,
@@ -114,7 +114,7 @@ def parse_args(args):
         choices=("xlrd", "openpyxl"),
         help="pandas XLSX reader engine",
     )
-    parser_convert.set_defaults(process=convert)
+    parser_export.set_defaults(process=export)
     # =========================================================================
     # parser_format:
     # inplace formatting
