@@ -13,7 +13,7 @@ import pytest
 
 from gridsource import Data as IVData
 from gridsource import ValidData as VData
-from gridsource.validation import DataFrameSchematizer
+from gridsource.validation import DataFrameSchematizer, ureg
 
 
 def test_DFS_00_schema_bulding():
@@ -89,7 +89,7 @@ def test_DFS_01_validation():
     |--------------+---------+-----------+-----------+-------|
     | blank or nan |         | True      | FAIL      | da    |
     | blank or nan |         | False     | blank nan | db    |
-    | blank or nan | 1       | True      | 1         | dc    |
+    | blank or nan | 1       | True      | FAIL      | dc    |
     | blank or nan | 1       | False     | 1         | dd    |
     | 2            |         | True      | 2         | de    |
     | 2            |         | False     | 2         | df    |
@@ -143,7 +143,10 @@ dh:
     )
     df, is_valid, errors = v.validate_dataframe(df)
     assert is_valid is False
-    expected_errors = {("da", 0): ["None is not of type 'integer'"]}
+    expected_errors = {
+        ("da", 0): ["None is not of type 'integer'"],
+        ("dc", 0): ["None is not of type 'integer'"],
+    }
     assert errors == expected_errors
     # check that values have been correctly filled
     expected_df = pd.DataFrame(
@@ -161,6 +164,83 @@ dh:
     assert pd.testing.assert_frame_equal(df, expected_df, check_dtype=False) is None
 
 
+def test_DFS_02_validation():
+    """test DataFrameSchematizer class validation.
+    specifically testing enums with default
+    """
+    columns_specs = """\
+---
+CA:
+  types: integer
+  default: 0
+  units: m
+CB:
+  types: number
+  default: 1.0
+CC:
+  types: number
+  default: 1.0
+CD:
+  types: number
+  default: 0
+  units: N
+CE:
+  types: string
+  enum:
+    - fwd
+    - aft
+  default: aft
+CF:
+  types: number
+  default: 0
+  units: kg
+"""
+    v = DataFrameSchematizer()
+    v.add_columns(columns_specs)
+    df = pd.DataFrame(
+        {
+            "CA": {0: "cm", 1: 5, 2: 15},
+            "CB": {0: "", 1: 5.3},  # empty units specified, but that's fine
+            "CC": {1: 5.3},  # no units specified, but that's fine
+            "CD": {1: 54.3},  # no units specified, that's an issue!
+            "CE": {1: "fwd"},
+            "CF": {0: "lbm", 1: 2},  # "lbm" doesn't exist
+        }
+    )
+    df, is_valid, errors = v.validate_dataframe(df)
+    assert errors == {
+        ("CD", None): ["no units specified in source file"],
+        ("CF", None): ["undefined units 'lbm'"],
+    }
+    # let's correct the previous errors
+    v = DataFrameSchematizer()
+    v.add_columns(columns_specs)
+
+    df = pd.DataFrame(
+        {
+            "CA": {0: "cm", 1: 5, 2: 15},
+            "CB": {0: "", 1: 5.3},  # empty units specified, but that's fine
+            "CC": {1: 5.3},  # no units specified, but that's fine
+            "CD": {0: "lbf", 1: 54.3},  # no units specified, that's an issue!
+            "CE": {1: "fwd"},
+            "CF": {0: "lb", 1: 2},  # "lbm" doesn't exist
+        }
+    )
+    df, is_valid, errors = v.validate_dataframe(df)
+    assert errors == {}
+    exp = pd.DataFrame(
+        {
+            "CA": {1: 0.05, 2: 0.15},
+            "CB": {1: 5.3, 2: 1.0},
+            "CC": {1: 5.3, 2: 1.0},
+            "CD": {1: 241.53843370864516, 2: 0.0},
+            "CE": {1: "fwd", 2: "aft"},
+            "CF": {1: 0.9071847400000002, 2: 0.0},
+        }
+    )
+    assert pd.testing.assert_frame_equal(df, exp) is None
+
+
 def test_VData_00():
     data = VData()
     # ------------------------------------------------------------------------
@@ -247,7 +327,7 @@ def test_IVData_00(datadir):
             assert is_ok is True
         except:
             print("OUPS!")
-            breakpoint()
+            pp(errors)
         else:
             print("OK")
         assert errors == {}
@@ -382,6 +462,50 @@ def test_IVData_05_failing_units(datadir):
         os.path.join(indir, "test_04_IMP.yaml"),
         os.path.join(indir, "test_04_data.yaml"),
     )
+    # -------------------------------------------------------------------------
+    # test tabname present in schemas, but not in data
+    whatever = data._schemas["whatever"].build()
+    assert whatever == {
+        "$schema": "http://json-schema.org/draft-07/schema#",
+        "properties": {
+            "force": {
+                "items": {"anyOf": [{"type": "integer"}, {"type": "null"}]},
+                "type": "array",
+                "uniqueItems": False,
+                "units": "lbf",
+            },
+            "length": {
+                "items": {
+                    "anyOf": [{"type": "integer"}, {"type": "null"}],
+                    "default": 0,
+                },
+                "type": "array",
+                "uniqueItems": False,
+                "units": "in",
+            },
+            "mass": {
+                "items": {"anyOf": [{"type": "integer"}, {"type": "null"}]},
+                "type": "array",
+                "uniqueItems": False,
+                "units": "lb",
+            },
+            "pressure": {
+                "items": {"anyOf": [{"type": "integer"}, {"type": "null"}]},
+                "type": "array",
+                "uniqueItems": False,
+                "units": "psi",
+            },
+            "volume": {
+                "items": {"anyOf": [{"type": "integer"}, {"type": "null"}]},
+                "type": "array",
+                "uniqueItems": False,
+                "units": "in^3",
+            },
+        },
+        "required": [],
+        "type": "object",
+    }
+
     expected_schema = {
         "$schema": "http://json-schema.org/draft-07/schema#",
         "properties": {
@@ -431,7 +555,7 @@ def test_IVData_05_failing_units(datadir):
     expected_report = {
         "mass": {
             ("test_mass_g", 2): ["None is not of type 'number'"],
-            ("test_mass_lbm", "*"): ["undefined units 'lbm'"],
+            ("test_mass_lbm", None): ["undefined units 'lbm'"],
         },
         "volume": {"general": ["'test_volume_foot3' is a required property"]},
     }
@@ -511,3 +635,66 @@ def test_IVData_06_units(datadir):
         df_imp = data_imp.convert_to(tabname)  # .pint.to_base_units()
         df_si = data_SI.convert_to(tabname)  # .pint.to_base_units()
         assert pd.testing.assert_frame_equal(df_imp, df_si) is None
+    # -------------------------------------------------------------------------
+    # getunits
+    assert data_SI.getunits("pressure", key="test_p_bar") == "MPa"
+    assert data_imp.getunits("pressure", key="test_p_bar") == "psi"
+    # column length/id has no units:
+    with pytest.raises(KeyError):
+        data_imp.getunits("length", key="id")
+    # getvalue
+    l9 = data_SI.getrow("length", key=9, search_by="id")
+    l9_expected = pd.Series(
+        {
+            "test_dist_m": 2000,
+            "test_dist_yd": 2000,
+            "test_dist_in": 2000,
+            "test_dist_ft": 2000,
+            "test_dist_foot": 2000,
+            "test_string": "a",
+            "groucho": 3.2,
+        },
+        name=9.0,
+    )
+    assert pd.testing.assert_series_equal(l9, l9_expected) is None
+
+    l9_value = data_SI.getvalue(
+        "length", key=9, search_by="id", column="test_dist_yd", as_quantity=False
+    )
+    assert pytest.approx(l9_value) == 2000
+
+    l9_quantity_exp = 2 * ureg.m
+    # rendering with quantity:
+    l9_quantity = data_SI.getvalue(
+        "length", key=9, search_by="id", column="test_dist_yd"
+    )
+    diff = l9_quantity - l9_quantity_exp
+    assert abs(diff.magnitude) < 1e-5
+
+
+def test_IVData_07_failing_units(datadir):
+    """check units"""
+    indir, outdir = datadir
+    data_si = IVData()
+    data_si.read(os.path.join(indir, "test_05_units_failing.xlsx"))
+    # -------------------------------------------------------------------------
+    # read and check schema
+    data_si.read_schema(
+        os.path.join(indir, "test_05_SI.yaml"),
+        os.path.join(indir, "test_05_data.yaml"),
+    )
+    rep = data_si.validate()
+    assert rep == {
+        "mass": {
+            "general": [
+                "'test_mass_tonne' is a required property",
+                "'test_mass_g' is a required property",
+            ],
+            ("test_mass_lb", 0): [
+                "'a' is not valid under any of the given " "schemas",
+                "'a' is not of type 'number'",
+                "'a' is not of type 'null'",
+            ],
+            ("test_mass_lb", "?"): "cannot convert some values to numeric",
+        }
+    }
